package com.yunzhi.niucai.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.yunzhi.niucai.common.DateUtils;
import com.yunzhi.niucai.entity.*;
import com.yunzhi.niucai.enums.SportOpeningStatusEnum;
import com.yunzhi.niucai.enums.SportStatusEnum;
import com.yunzhi.niucai.mapper.TaMatchMapper;
import com.yunzhi.niucai.mapper.TaMatchOddsMapper;
import com.yunzhi.niucai.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 比赛  服务实现类
 * </p>
 *
 * @author yansen
 * @since 2020-09-02
 */
@Service
public class TaMatchServiceImpl extends ServiceImpl<TaMatchMapper, TaMatch> implements ITaMatchService {

    @Autowired
    ITaMatchWayService iTaMatchWayService;

    @Autowired
    ITaMatchOddsService iTaMatchOddsService;

    @Autowired
    ITaCustomerBettingService iTaCustomerBettingService;

    @Autowired
    ITaCustomerBettingItemService iTaCustomerBettingItemService;

    @Autowired
    ITaCustomerBettingSportService iTaCustomerBettingSportService;

    @Autowired
    TaMatchMapper taMatchMapper;

    @Autowired
    TaMatchOddsMapper taMatchOddsMapper;

    @Override
    public List<TaMatch> getMixedMatch(String lottery) throws Exception {
        List<TaMatch> taMatches = taMatchMapper.getMixedMatch(lottery, SportStatusEnum.SALE.getCode());
        if (null == taMatches) {
            return null;
        }

        for (TaMatch match : taMatches) {
            List<Map<String, Object>> oddList = taMatchOddsMapper.getByMatch(match.getMatchId());
            if (null != oddList) {
                Map<String, List<Map<String, Object>>> odds = new HashMap<>();

                String key = "";
                List<Map<String, Object>> list = new ArrayList<>();
                for (Map<String, Object> item : oddList) {
                    String wayCode = item.get("wayCode").toString();
                    if (!key.equals(wayCode)) {
                        key = wayCode;
                        list = new ArrayList<>();
                        odds.put(key, list);
                    }
                    list.add(item);
                }

                Map<String, TaMatchWay> groupedByWay = iTaMatchWayService.getGroupedByWay(match.getMatchId(), lottery);

                match.setPlayOdds(odds);
                match.setMatchWay(groupedByWay);
            }
        }

        return taMatches;
    }

    @Override
    public List<TaMatch> getLastResult() throws Exception {
        return taMatchMapper.getLastResult();
    }

    /**
     * 体彩是否中奖
     * @throws Exception
     */
    public void checkAndSetSportPrize() throws Exception {
        List<TaMatch> matchList = taMatchMapper.getLastOpenAndUndoMatch(3);
        if (null == matchList || matchList.size() == 0) {
            return;
        }

        for (TaMatch taMatch : matchList) {
            try {
                taMatch.setOpeningStatus(SportOpeningStatusEnum.DOING.getCode());
                updateById(taMatch);

                String lotteryId = taMatch.getLotteryId();
                switch (lotteryId) {
                    case "football":
                        setFootballPrize(taMatch);
                        break;
                    case "basketball":
                        setBasketballPrize(taMatch);
                        break;
                    default:
                        break;
                }

            } catch (Exception e) {
                // TODO 比如错误日志记录
                taMatch.setOpeningStatus(SportOpeningStatusEnum.READY.getCode());
                updateById(taMatch);
            }
        }
    }

    /**
     * 足球开奖
     * @param taMatch
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    private void setFootballPrize(TaMatch taMatch) throws Exception {
        // 获取当前比赛的投注列表
        List<TaCustomerBettingItem> bettingItemList = iTaCustomerBettingItemService.getByMatch(taMatch);
        if (null == bettingItemList || bettingItemList.size() == 0) {
            return;
        }

        for (TaCustomerBettingItem item: bettingItemList) {
            if (!SportOpeningStatusEnum.READY.getCode().equals(item.getOpeningStatus())) {
                continue;
            }

            // 通过明细反向查询主记录
            // 但是主记录包含的明细不止当前这条
            TaCustomerBetting betting = iTaCustomerBettingService.getById(item.getBettingId());
            // 投注列表
            List<TaCustomerBettingSport> bettingSportList = iTaCustomerBettingSportService.getListByBetting(item.getBettingId(), item.getItemNo());
            // 判断是否中奖
            for (TaCustomerBettingSport bettingSport : bettingSportList) {
                boolean wining = isWining(taMatch, bettingSport);
                if (wining) {
                    bettingSport.setIsWinning(wining);
                    iTaCustomerBettingSportService.updateById(bettingSport);
                }
            }

            item.setOpeningStatus(SportOpeningStatusEnum.FINISHED.getCode());
            item.setSportList(bettingSportList);

            // 验证当前明细对应的主记录的所有明细是否都已经处理了
            checkAndSetBetting(betting, item);
        }
    }

    private void checkAndSetBetting(TaCustomerBetting betting, TaCustomerBettingItem item) throws Exception {
        List<TaCustomerBettingItem> itemList = iTaCustomerBettingItemService.getByBetting(betting);
        if (null == itemList || itemList.size() == 0) {
            throw new Exception(String.format("投注[%s]没有找到任何投注明细", betting.getBettingId()));
        }

        boolean done = true;
        for (TaCustomerBettingItem bettingItem : itemList) {
            if (bettingItem.getItemNo().equals(item.getItemNo())) {
                bettingItem = item;
                continue;
            }

            // 如果还有未处理完成的, 跳出循环
            if (!SportOpeningStatusEnum.FINISHED.getCode().equals(bettingItem.getOpeningStatus())) {
                done = false;
                break;
            }
        }

        // 如果还有场次没有结束, 那么不去算最终奖金
        if (!done) {
            return;
        }

        // 结算最终奖金
        computePirzeAmount(betting, itemList);
    }

    /**
     * 结算最终奖金
     * 主要思路：
     * 1、先把所有场次组合一个数组, 其实就是 itemList
     * 2、依据所选的玩法, 把所有的玩法都组合出来, 比如4串5, 那么用场次把所有组合都穷举出来
     * 3、遍历所有组合, 计算每种组合的中奖金额
     * @param betting
     * @param itemList
     * @throws Exception
     */
    private void computePirzeAmount(TaCustomerBetting betting, List<TaCustomerBettingItem> itemList) throws Exception {
        // 参与算法的数据结构是一个两层List
        // List<                场次, 也就是最外层的 size == 场次数
        //      List<           投注名称, 内层的 size == 每场投注数
        //          String>>    当前投注的打单赔率, 不是投注赔率
        List<List<String>> noteList = new ArrayList<>();

        // 获取每个场次的玩法投注列表
        for (TaCustomerBettingItem item : itemList) {
            List<TaCustomerBettingSport> sportList = item.getSportList();

            if (null == sportList || sportList.size() == 0) {
                sportList = iTaCustomerBettingSportService.getListByBetting(betting.getBettingId(), item.getItemNo());
                if (null == sportList || sportList.size() == 0) {
                    continue;
                }
            }

            // 只有中奖的投注才会参与计算
            // 所以会出现 size = 0 情况
            List<String> oddsOfNote = new ArrayList<>();
            for (TaCustomerBettingSport sportItem : sportList) {
                if (sportItem.getIsWinning()) {
                    oddsOfNote.add(sportItem.getOddsPrinted());
                }
            }

            noteList.add(oddsOfNote);
        }
    }

    /**
     * 篮球开奖
     * @param taMatch
     * @throws Exception
     */
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    private void setBasketballPrize(TaMatch taMatch) throws Exception {

    }

    private boolean isWining(TaMatch taMatch, TaCustomerBettingSport bettingSport) {
        TaMatchOdds taMatchOdds = iTaMatchOddsService.getByRule(taMatch.getMatchId(), bettingSport.getRuleCode());
        return null != taMatchOdds && taMatchOdds.getIsWinning();
    }
}
